﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using ESBasic.ObjectManagement.Forms;
using StriveEngine.Tcp.Passive;
using ESBasic;
using Core;
using ESBasic.Helpers;

namespace StriveEngineChatDemo
{
    public partial class ChatForm: Form, IManagedForm<string>
    {
        private string friendID;
        private string selfUserID;
        private MainForm mainForm;
        private ITcpPassiveEngine tcpPassiveEngine;

        public string FormID
        {
            get { return this.friendID; }
        }

        public ChatForm(string _selfUserID, string _friendID, MainForm _mainForm,ITcpPassiveEngine _tcpPassiveEngine)
        {
            InitializeComponent();            
            this.selfUserID = _selfUserID;
            this.friendID = _friendID;
            this.mainForm = _mainForm;
            this.tcpPassiveEngine = _tcpPassiveEngine;
            this.mainForm.ChatMsgReceived += new CbGeneric<ChatMsg>(this.formMain_chatMsgReceived);
        }

        #region 接收消息的处理
        void formMain_chatMsgReceived(ChatMsg chatMsg)
        {
            this.ShowChatMsg(chatMsg);
        } 
        #endregion

        #region 聊天消息的发送与显示
        /// <summary>
        /// 显示聊天消息
        /// </summary>
        /// <param name="chatMsg"></param>
        private void ShowChatMsg(ChatMsg chatMsg)
        {
            if (this.InvokeRequired)
            {
                this.Invoke(new CbGeneric<ChatMsg>(this.formMain_chatMsgReceived), chatMsg);
            }
            else
            {
                this.richTextBox_display.AppendText(chatMsg.SourceUserID + "   " + chatMsg.TimeSent.ToString() + "\r\n");
                this.richTextBox_display.AppendText(chatMsg.MsgText + "\r\n");
                this.richTextBox_Write.Clear();
            }
        }
        /// <summary>
        /// 发送聊天消息
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void button_send_Click(object sender, EventArgs e)
        {
            string chatText = this.richTextBox_Write.Text;
            if (string.IsNullOrEmpty(chatText))
            {
                MessageBox.Show("消息不能为空");
                return;
            }
            ChatMsg chatMsg = new ChatMsg(this.selfUserID, this.friendID, chatText);
            this.tcpPassiveEngine.SendMessageToServer(chatMsg.ToContractStream());
            this.ShowChatMsg(chatMsg);
        }       
        #endregion

        #region 会话的中止与恢复
        /// <summary>
        /// 恢复会话
        /// </summary>
        public void RecoverConversation()
        {
            if (this.InvokeRequired)
            {
                this.Invoke(new CbGeneric(this.RecoverConversation), null);
            }
            else
            {
                this.richTextBox_display.AppendText(DateTime.Now.ToString() + "\r\n");
                this.richTextBox_display.AppendText(this.friendID + "已经上线!" + "\r\n");
                this.button_send.Enabled = true;
            }
        }
        /// <summary>
        /// 中止会话
        /// </summary> 
        public void AbortConversation()
        {
            if (this.InvokeRequired)
            {
                this.Invoke(new CbGeneric(this.AbortConversation), null);
            }
            else
            {
                this.richTextBox_display.AppendText(DateTime.Now.ToString() + "\r\n");
                this.richTextBox_display.AppendText(this.friendID + "已经掉线!" + "\r\n");
                this.button_send.Enabled = false;
            }
        }
        #endregion       

        private void ChatForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            this.mainForm.ChatMsgReceived -= new CbGeneric<ChatMsg>(this.formMain_chatMsgReceived);
        }

        private void richTextBox_display_TextChanged(object sender, EventArgs e)
        {
            this.richTextBox_display.ScrollToCaret();
        }
    }
}
