﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using StriveEngine.Tcp.Passive;
using Core;
using System.Net;
using ESBasic;
using ESBasic.ObjectManagement.Managers;
using ESBasic.ObjectManagement.Forms;
using ESBasic.Helpers;

namespace StriveEngineChatDemo
{
    public partial class MainForm : Form
    {
        private ITcpPassiveEngine tcpPassiveEngine;
        private string selfID;
        private List<string> listFriendUser = new List<string>();//好友列表       
        private FormManager<string, ChatForm> chatFormManager = new FormManager<string, ChatForm>();//用于管理与其他用户的聊天窗口
        /// <summary>
        /// 当收到聊天消息时，告知chatForm    
        /// </summary>
        public event CbGeneric<ChatMsg> ChatMsgReceived;
        /// <summary>
        /// 构造方法
        /// </summary>
        /// <param name="_tcpPassiveEngine"></param>
        /// <param name="_selfID"></param>
        public MainForm(ITcpPassiveEngine _tcpPassiveEngine, string _selfID)
        {
            InitializeComponent();
            this.tcpPassiveEngine = _tcpPassiveEngine;
            this.selfID = _selfID;
            this.tcpPassiveEngine.MessageReceived += new StriveEngine.CbDelegate<IPEndPoint, byte[]>(tcpPassiveEngine_MessageReceived);
            this.tcpPassiveEngine.ConnectionInterrupted += new StriveEngine.CbDelegate(tcpPassiveEngine_ConnectionInterrupted);
            this.tcpPassiveEngine.ConnectionRebuildSucceed += new StriveEngine.CbDelegate(tcpPassiveEngine_ConnectionRebuildSucceed);
        }

        #region 断线重连
        /// <summary>
        /// 重连成功事件处理
        /// </summary>
        void tcpPassiveEngine_ConnectionRebuildSucceed()
        {
            LoginMsg loginMsg = new LoginMsg(this.selfID);
            this.tcpPassiveEngine.PostMessageToServer(loginMsg.ToContractStream());
        }
        /// <summary>
        /// 掉线事件处理
        /// </summary>
        void tcpPassiveEngine_ConnectionInterrupted()
        {
            this.BeingOffline();
        } 
        #endregion

        #region 接收消息处理
        /// <summary>
        /// 消息接收事件处理
        /// </summary>
        /// <param name="userAddress">消息源用户IP端口</param>
        /// <param name="msg">消息</param>
        void tcpPassiveEngine_MessageReceived(IPEndPoint userAddress, byte[] msg)
        {
            //取出消息类型
            int msgType = BitConverter.ToInt32(msg, 0);
            //验证消息类型 
            if (msgType == MsgType.Logining)
            {
                LoginMsg loginMsg = MsgHelper.DeserializeMsg<LoginMsg>(msg);
                if (loginMsg.LogonResult == LogonResult.Succeed)
                {
                    this.BeingOnline(true);
                    this.RequestFriendList();
                }
                if (loginMsg.LogonResult == LogonResult.Repetition)
                {
                    MessageBox.Show("重登录失败，该账号已经登录！");
                }
            }
            if (msgType == MsgType.InitializeFriendList)
            {
                FriendListMsg friendListMsg = MsgHelper.DeserializeMsg<FriendListMsg>(msg);
                //加载好友列表        
                foreach (string id in friendListMsg.FriendList)
                {
                    this.AddFriend(id);
                }
            }
            if (msgType == MsgType.NewOnlineFriend)
            {
                string newUserID = Encoding.UTF8.GetString(msg, 8, msg.Length - 8);
                this.AddFriend(newUserID);
            }
            if (msgType == MsgType.FriendOffline)
            {
                string offUserID = Encoding.UTF8.GetString(msg, 8, msg.Length - 8);
                this.RemoveFriend(offUserID);
            }
            if (msgType == MsgType.Chatting)
            {
                ChatMsg chatMsg = MsgHelper.DeserializeMsg<ChatMsg>(msg);
                this.ShowChatForm(chatMsg.SourceUserID);
                this.ChatMsgReceived(chatMsg);
            }
        } 
        #endregion     

        #region 离、在线状态
        /// <summary>
        /// 将UI置为在线状态
        /// </summary>
        /// <param name="isConnectionRebuilt">本次状态设置是否是在断线重连的情景下？</param>
        private void BeingOnline(bool isConnectionRebuilt)
        {           
           
            if (this.InvokeRequired)
            {
                this.Invoke(new CbGeneric<bool>(this.BeingOnline), isConnectionRebuilt);
            }
            else
            {
                this.toolStripLabel_currentUser.Text = string.Format("当前登录：{0}", this.selfID);
                if (isConnectionRebuilt)
                {
                    this.toolStripLabel_State.Text = "连接状态：正常(重连成功)";
                }
                else
                {
                    this.toolStripLabel_State.Text = "连接状态：正常";
                }
                this.toolStripLabel_State.ForeColor = Color.Black;
            }
        }
        /// <summary>
        /// 将UI置为离线状态
        /// </summary>
        private void BeingOffline()
        {
            if (this.InvokeRequired)
            {
                this.Invoke(new CbGeneric(this.BeingOffline), null);
            }
            else
            {                
                this.toolStripLabel_State.Text = "连接状态：断开，重连中......";
                this.toolStripLabel_State.ForeColor = Color.Red;
            }
        } 
        #endregion

        #region 好友添加与删除
        /// <summary>
        /// 添加好友
        /// </summary>
        /// <param name="friendID">好友ID</param>
        private void AddFriend(string friendID)
        {
            if (!this.listFriendUser.Contains(friendID))
            {
                this.listFriendUser.Add(friendID);
            }
            this.AddItem(friendID);
            if (this.chatFormManager.Contains(friendID))
            {
                this.chatFormManager.GetForm(friendID).RecoverConversation();
            }
        }
        /// <summary>
        /// 移除好友
        /// </summary>
        /// <param name="offID">好友ID</param>
        private void RemoveFriend(string offID)
        {
            this.listFriendUser.Remove(offID);
            this.RemoveItem(offID);
            if (this.chatFormManager.Contains(offID))
            {
                this.chatFormManager.GetForm(offID).AbortConversation();
            }
        }
        /// <summary>
        /// 添加好友的UI显示
        /// </summary>
        /// <param name="friendID">好友ID</param>
        private void AddItem(string friendID)
        {
            if (this.InvokeRequired)
            {
                this.Invoke(new CbGeneric<string>(this.AddItem), friendID);
            }
            else
            {
                this.listView1.Items.Add(friendID, friendID, 0);
            }
        }
        /// <summary>
        /// 移除好友的UI显示
        /// </summary>
        /// <param name="friendID">好友ID</param>
        private void RemoveItem(string friendID)
        {
            if (this.InvokeRequired)
            {
                this.Invoke(new CbGeneric<string>(this.RemoveItem), friendID);
            }
            else
            {
                this.listView1.Items.RemoveByKey(friendID);
            }
        } 
        #endregion

        /// <summary>
        /// 显示聊天窗
        /// </summary>
        /// <param name="friendUserID">聊天对方用户ID</param>
        private void ShowChatForm(string friendUserID)
        {
            if (this.InvokeRequired)
            {
                this.Invoke(new CbGeneric<string>(this.ShowChatForm), friendUserID);
            }
            else
            {
                ChatForm form = this.chatFormManager.GetForm(friendUserID);
                if (form == null)
                {
                    form = new ChatForm(this.selfID, friendUserID, this, this.tcpPassiveEngine);
                    form.Text = string.Format("与{0}对话中···", friendUserID);
                    this.chatFormManager.Add(form);
                    form.Show();
                }
                form.Focus();
            }
        }
        /// <summary>
        /// 主窗完成显示事件处理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_Shown(object sender, EventArgs e)
        {
            this.BeingOnline(false);
            this.RequestFriendList();
        }
        /// <summary>
        /// 请求获取好友列表
        /// </summary>
        private void RequestFriendList()
        {
            FriendListMsg friendListMsg = new FriendListMsg(this.selfID);
            this.tcpPassiveEngine.SendMessageToServer(friendListMsg.ToContractStream());
        }
        /// <summary>
        /// 双击listView中的显示项事件处理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void listView1_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            if (e.Button != System.Windows.Forms.MouseButtons.Left)
            {
                return;
            }
            ListViewHitTestInfo info = this.listView1.HitTest(e.Location);
            if (info.Item != null)
            {
                this.ShowChatForm(info.Item.Text);
            }
        }

    }
}

